/**
 * @file FaspManagerException.h
 * 
 * Aspera, Inc.
 */

#ifndef __FASP_MANAGER_EXCEPTION_H__
#define __FASP_MANAGER_EXCEPTION_H__

#include "FaspManagerCommon.h"
#include <exception>
#include <string>

namespace Aspera
{
namespace FaspManager
{

/// A class providing a base for FASP management related exception handling.
class Exception
{
  /* Member variables */
  protected:

    /// The error message associated with this FASP exception
    std::string m_err_msg;

  /* Member functions */
  public:

    /// Constructs this default FASP exception.
    FASP_MANAGER_API Exception( ) : m_err_msg( "Generic FASP exception" ) { };


    /// Constructs this FASP exception.
    ///
    /// @param err_msg the error message associated with this FASP exception
    FASP_MANAGER_API Exception( std::string err_msg ) : m_err_msg( err_msg ) { };

    /// Deconstructs this FASP exception.
    FASP_MANAGER_API virtual ~Exception( ) { };

    /// Gets the error message associated with this FASP exception.
    ///
    /// @return the error message associated with this FASP exception
    FASP_MANAGER_API const char* GetErrMsg( ) { return m_err_msg.c_str(); };
};


/// A class to define a FASP Manager socket related execption.
class SocketException : public Exception
{
  /* Member functions */
  public:

    /// Constructs this default FASP socket exception.
    FASP_MANAGER_API SocketException( ) : Exception( ) { };


    /// Constructs this FASP socket exception.
    ///
    /// @param err_msg the error message associated with this FASP socket exception
    FASP_MANAGER_API SocketException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP socket exception.
    FASP_MANAGER_API ~SocketException( ) { };
};

/// A class to define a FASP Manager transfer related execption.
class XferException : public Exception
{
  /* Member functions */
  public:
    /// Constructs this default FASP transfer exception.
    FASP_MANAGER_API XferException( ) : Exception( ) { };

    /// Constructs this FASP transfer exception.
    ///
    /// @param err_msg the error message associated with this FASP transfer exception

    FASP_MANAGER_API XferException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP transfer exception.
    FASP_MANAGER_API ~XferException( ) { };
};

/// A class to define an invalid API operation.
class InvalidOperationException : public Exception
{
  /* Member functions */
  public:
    /// Constructs this default FASP state exception.
    FASP_MANAGER_API InvalidOperationException( ) : Exception( ) { };

    /// Constructs this FASP state exception.
    ///
    /// @param err_msg the error message associated with this FASP state
    /// exception
    FASP_MANAGER_API InvalidOperationException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP state exception.
    FASP_MANAGER_API ~InvalidOperationException( ) { };
};

/// A class to define a FASP Manager job not found related execption.
class JobNotFoundException : public Exception
{
  /* Member functions */
  public:

    /// Constructs this default FASP job not found exception.
    FASP_MANAGER_API JobNotFoundException( ) : Exception( ) { };

    /// Constructs this FASP job not found exception.
    ///
    /// @param err_msg the error message associated with this FASP job not found
    /// exception
    FASP_MANAGER_API JobNotFoundException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP job not found exception.
    FASP_MANAGER_API ~JobNotFoundException( ) { };
};


/// A class to define a FASP Manager send related exception.
class SendException : public Exception
{
  /* Member functions */
  public:

    /// Constructs this default FASP send exception.
    FASP_MANAGER_API SendException( ) : Exception( ) { };

    /// Constructs this FASP send exception.
    ///
    /// @param err_msg the error message associated with this FASP send
    /// exception
    FASP_MANAGER_API SendException( std::string err_msg ) : Exception( err_msg ) { };


    /// Deconstructs this FASP send exception.
    FASP_MANAGER_API ~SendException( ) { };
};

/// A class to define a FASP state related exception.
class StateException : public Exception
{
  /* Member functions */
  public:
    /// Constructs this default FASP state exception.
    FASP_MANAGER_API StateException( ) : Exception( ) { };

    /// Constructs this FASP state exception.
    ///
    /// @param err_msg the error message associated with this FASP state
    /// exception
    FASP_MANAGER_API StateException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP state exception.
    FASP_MANAGER_API ~StateException( ) { };
};

/// A class to define a FASP Manager validation exception.
class ValidationException : public Exception
{
  /* Member functions */
  public:

    /// Constructs a default validation exception.
    FASP_MANAGER_API ValidationException( ) : Exception( ) { };

    /// Constructs a validation exception.
    ///
    /// @param err_msg the error message associated with failed validation
    /// exception
    FASP_MANAGER_API ValidationException( std::string err_msg ) : Exception( err_msg ) { };

    /// Deconstructs this FASP send exception.
    FASP_MANAGER_API ~ValidationException( ) { };
};

} /* namespace FaspManager */
} /* namespace Aspera */

#endif /* __FASP_MANAGER_EXCEPTION_H__ */
