/**
 * @file FaspManagerFunctor.h
 *
 * Aspera, Inc.
 */

#ifndef __FASP_MANAGER_FUNCTOR_H__
#define __FASP_MANAGER_FUNCTOR_H__

#include "FaspManagerCommon.h"

namespace Aspera
{
namespace FaspManager
{

/* Forward declarations */
struct JobStats;
struct FileStats;


/// A class providing an abstract base for FASP functors.
class Functor
{
  /* Member functions */
  public:
    /// Destructs this FASP functor.
    virtual ~Functor( ) { };

    /// Clones this FASP functor.
    ///
    /// \return a clone of this FASP functor
    virtual Functor *Clone( ) = 0;

    /// Overloads the "()" operator (the function call operator) for this
    /// FASP functor.
    ///
    /// \param cb_event the event on which the callback is performed
    /// \param j_stats the FASP Job stats
    /// \param f_stats the current FASP Job file stats
    virtual void operator()(callback_event_e cb_event, const JobStats* j_stats, const FileStats* f_stats ) = 0;

    /// Compare this instance of Functor with the parameter
    /// \param other instance of function to compare with
    //  \return true if the two objects are the same
    virtual bool CompareWith(const Functor* other) = 0;
};

/// A class providing a functor for use in FASP management callbacks. It is required that the implementation 
/// of the callback function return quickly. Any time consuming tasks must be performed on a different thread.
template <class T> class CallbackFunctor : public Functor
{
  /* Member variables */
  private:
    T *m_obj_ptr;  /* the pointer to the object */
                   /* the pointer to the member function */
    void (T::*m_fnct_ptr)(callback_event_e cb_event, const JobStats*, const FileStats* );

  /* Member functions */
  public:
    /// Constructs this FASP callback functor.
    ///
    /// \param obj the object used in the callback
    /// \param fnct the function pointer used in the callback
    FASP_MANAGER_API CallbackFunctor( T *obj, void (T::*fnct)(callback_event_e cb_event, const JobStats*, const FileStats* ) ):
      m_obj_ptr( obj ),
      m_fnct_ptr( fnct )
    { };

    /// Copy constructs this FASP callback functor.
    ///
    /// \param functor the Fasp callback functor to be copy constructed
    FASP_MANAGER_API CallbackFunctor( CallbackFunctor &functor )
    : Functor()
    {
      m_obj_ptr = functor.m_obj_ptr;
      m_fnct_ptr = functor.m_fnct_ptr;
    };

    /// Destructs this FASP callback functor.
    FASP_MANAGER_API ~CallbackFunctor( ) { };

    /// Clones this FASP callback functor.
    ///
    /// \return a clone of this FASP callback functor
    FASP_MANAGER_API Functor *Clone( )
    {  return new CallbackFunctor( *this );  };

    /// Overloads the "()" operator (the function call operator) for this FASP
    /// callback functor.
    ///
    /// \param cb_event the event on which the callback is performed 
    /// \param j_stats the FASP Job stats
    /// \param f_stats the current FASP Job file stats
    FASP_MANAGER_API void operator()(callback_event_e cb_event, const JobStats* j_stats, const FileStats* f_stats )
    {  (m_obj_ptr->*m_fnct_ptr)(cb_event, j_stats, f_stats );  };

    FASP_MANAGER_API bool CompareWith(const Functor* other) 
    { 
        if (const CallbackFunctor* cbfOther = dynamic_cast<const CallbackFunctor*>(other)) {
            return (m_obj_ptr == cbfOther->m_obj_ptr) && (m_fnct_ptr == cbfOther->m_fnct_ptr);
        }
        return false;
    }
};


} /* namespace FaspManager */
} /* namespace Aspera */

#endif /* __FASP_MANAGER_FUNCTOR_H__ */
